;****************************************************************************
; RadioPOP is a TSR control program for the RadioDRV TSR program.  It lets
; you change the base frequency, slew the base frequency up and down, toggle
; frequency updating of the xcvr/rcvr, and toggle "one_shot" frequency
; updating.  Its syntax is
;
;       RADIOPOP [/M]
;
; where /M runs the program in monochrome mode on color video adapters --
; useful for lap-tops with color adapters but LCD or gas plasma screens.
; Once RadioPOP is installed, pressing Alt-Rt Shift pops it up.
;****************************************************************************
;
; Note:  Interrupt 64h to RadioDRV always has register AH >= 16.  OrbitDRV
;        issues interrupt 64h with AH = 3 to indicate a range-rate change.
;

border_color    equ     [bx]                    ;Color table equates
window_color_1  equ     [bx+1]
window_color_2  equ     [bx+2]
hilite_color_1  equ     [bx+3]
hilite_color_2  equ     [bx+4]
shadow_color    equ     [bx+5]
menu_color1     equ     [bx+6]
menu_color2     equ     [bx+7]
title_color     equ     [bx+8]
error_color     equ     [bx+9]
input_color     equ     [bx+10]

;****************************************************************************
; Program code and data.
;****************************************************************************

code            segment
                assume  cs:code
                org     100h
begin:          jmp     init

signature       db      0,1,2,"RadioPOP",2,1,0  ;Program signature
signature1      db      "RadioDRV"
prog_id         db      ?                       ;Multiplex ID number

int09h          dd      ?                       ;Interrupt 09H vector
int10h          dd      ?                       ;Interrupt 10H vector
int1Bh          dd      ?                       ;Interrupt 1Bh vector
int23h          dd      ?                       ;Interrupt 23H vector
int24h          dd      ?                       ;Interrupt 24H vector
int28h          dd      ?                       ;Interrupt 28H vector
int2Fh          dd      ?                       ;Interrupt 2FH vector

dosversion      dw      ?                       ;DOS version number
inbuffer        db      128 dup (0)             ;String input buffer
egaflag         db      1                       ;1=EGA/VGA/XGA installed
popupflag       db      0                       ;Non-zero if window is up
requestflag     db      0                       ;Non-zero if hotkey pressed
errorflag       dd      ?                       ;Critical error flag address
indos           dd      ?                       ;InDOS flag address
intflags        db      0                       ;User interrupt flags
oldpsp          dw      ?                       ;Active process ID
ss_register     dw      ?                       ;Contents of SS register
sp_register     dw      ?                       ;Contents of SP register

color_table     db      1Bh,3Fh,1Fh,1Eh         ;Color video attributes
                db      4Fh,07h,7Fh,70h
                db      1Fh,4Fh,02h

mono_table      db      70h,07h,70h,70h         ;Monochrome video attributes
                db      07h,07h,0Fh,0Fh
                db      70h,70h,07h

color_ptr       dw      offset color_table      ;Pointer to color table
video_segment   dw      0B800h                  ;Video buffer segment
video_offset    dw      ?                       ;Video buffer offset
small_cursor    dw      0607h                   ;Small cursor shape
big_cursor      dw      0407h                   ;Bug cursor shape
line_length     dw      ?                       ;Bytes per video line
cursor_mode     dw      ?                       ;Cursor mode
cursor_pos      dw      ?                       ;Cursor position
video_page      db      ?                       ;Video page number
window_start    label   word
window_x        db      44                      ;Starting column of window
window_y        db      1                       ;Starting row of window
columns         dw      0                       ;Number of columns in window
rows            dw      0                       ;Number of rows in window

title1          db      "    RadioDRV Control v 1.1    ",0
line            db      "",0
head1           db      "Offset    :                   ",0
head2           db      "Base Freq :                   ",0
head3           db      "Doppler   :                   ",0
head4           db      "Cur Freq  :                   ",0

funkey1         db      "F1        Slew      -10 Hz    ",0
funkey2         db      "F2                  +10 Hz    ",0
funkey3         db      "F3                 -100 Hz    ",0
funkey4         db      "F4                 +100 Hz    ",0
funkey5         db      "F5                -1000 Hz    ",0
funkey6         db      "F6                +1000 Hz    ",0
funkey7         db      "F7,  =    Zero Offset         ",0
funkey8         db      "F8,  U    Toggle Freq Updating",0
funkey9         db      "F9,  D    Toggle Band Display ",0
funkey10        db      "F10, /    Set Base Frequency  ",0

inputline       db      "New Base Frequency: "
freqstring      db      11 dup (0)

index           db      0                       ;Menu window index

xerror_ax       dw      ?                       ;Storage array for
xerror_bx       dw      ?                       ;  extended error
xerror_cx       dw      ?                       ;  information
xerror_dx       dw      ?
xerror_si       dw      ?
xerror_di       dw      ?
xerror_ds       dw      ?
xerror_es       dw      ?
                dw      3 dup (0)

;****************************************************************************
; KB_INT handles interrupt 09H.
;****************************************************************************

kb_int          proc    far
                pushf                           ;Push FLAGS
                call    cs:[int09h]             ;Call previous handler

                cmp     cs:[popupflag],1        ;Exit now if POPUPFLAG
                je      kb_exit                 ;  is already set

                push    ax                      ;Save AX and ES
                push    es
                mov     ax,40h                  ;Point ES to the BIOS
                mov     es,ax                   ;  Data Area
                mov     al,es:[17h]             ;Get keyboard flags
                and     al,0Fh                  ;Zero the upper 4 bits
                cmp     al,09h                  ;Is Alt-Rt Shift pressed?
                pop     es                      ;Restore AX and ES
                pop     ax
                jne     kb_exit                 ;No, then branch

                push    di                      ;Save DI and ES
                push    es
                cmp     cs:[intflags],0         ;Exit if a user interrupt
                jne     kb1                     ;  flag is set
                les     di,cs:[indos]           ;Exit if the InDOS flag
                cmp     byte ptr es:[di],0      ;  is non-zero
                jne     kb1
                les     di,cs:[errorflag]       ;Exit if the critical error
                cmp     byte ptr es:[di],0      ;  flag is non-zero
                jne     kb1
                mov     cs:[requestflag],0      ;Clear REQUESTFLAG
                mov     cs:[popupflag],1        ;Set POPUPFLAG
                call    popup                   ;Pop up the window
                mov     cs:[popupflag],0        ;Clear POPUPFLAG
                jmp     short kb2               ;Exit

kb1:            mov     requestflag,18          ;Set request flag
kb2:            pop     es                      ;Restore DI and ES
                pop     di
kb_exit:        iret                            ;Return from interrupt
kb_int          endp

;****************************************************************************
; VIDEO_INT handles interrupt 10H.
;****************************************************************************

video_int       proc    far
                pushf                           ;Save FLAGS
                or      cs:[intflags],01h       ;Set video interrupt flag
                popf                            ;Retrieve FLAGS
                pushf                           ;Push FLAGS
                call    cs:[int10h]             ;Call previous handler
                pushf                           ;Save FLAGS
                and     cs:[intflags],0FEh      ;Clear video interrupt flag
                popf                            ;Retrieve FLAGS
;                ret     2                       ;Return with FLAGS intact
                iret
video_int       endp

;****************************************************************************
; ERROR_INT handles interrupt 24H (and 1BH and 23H).
;****************************************************************************

error_int       proc    far
                mov     al,03h                  ;Fail the call
error_exit:     iret                            ;Return from interrupt
error_int       endp

;****************************************************************************
; DOSIDLE_INT handles interrupt 28H.
;****************************************************************************

dosidle_int     proc    far
                pushf                           ;Push FLAGS
                call    cs:[int28h]             ;Call previous handler

                cmp     cs:[requestflag],0      ;Exit if REQUESTFLAG is
                je      dosidle_exit            ;  not set

                push    di                      ;Save DI and ES
                push    es
                cmp     cs:[intflags],0         ;Exit if a user interrupt
                jne     dosidle2                ;  flag is set
                les     di,cs:[errorflag]       ;Exit if the critical error
                cmp     byte ptr es:[di],0      ;  flag is non-zero
                jne     dosidle2
                mov     cs:[requestflag],0      ;Clear REQUESTFLAG
                mov     cs:[popupflag],1        ;Set POPUPFLAG
                call    popup                   ;Pop up the window
                mov     cs:[popupflag],0        ;Clear POPUPFLAG
dosidle2:       pop     es                      ;Restore DI and ES
                pop     di
dosidle_exit:   iret                            ;Return from interrupt
dosidle_int     endp

;****************************************************************************
; MPLEX_INT handles interrupt 2FH.  If, on entry, AH is set to RadioPOP's
; multiplex ID number, MPLEX_INT uses the value in AL as a function code.
; The functions supported are:
;
;    00H    Returns FFH in AL to indicate the program is installed
;           and returns the address of its signature in ES:DI.
;****************************************************************************

mplex_int       proc    far
                pushf                           ;Save FLAGS register
                cmp     ah,cs:[prog_id]         ;Branch if AH holds the
                je      mplex2                  ;  multiplex ID
                popf                            ;Restore FLAGS
                jmp     cs:[int2Fh]             ;Pass the interrupt on
;
; Function 00H verifies that the program is installed.
;               
mplex2:         popf                            ;Restore FLAGS
                or      al,al                   ;Branch if function code
                jnz     mplex3                  ;  is other than 00H
                mov     al,0FFh                 ;Set AL to FFH
                push    cs                      ;Point ES:DI to the program
                pop     es                      ;  signature
                mov     di,offset signature
mplex3:         iret                            ;Return from interrupt
mplex_int       endp

;****************************************************************************
; POPUP pops up the RadioPOP window.
;****************************************************************************

popup           proc    near
                cli                             ;Disable interrupts
                mov     cs:[ss_register],ss     ;Save the SS register
                mov     cs:[sp_register],sp     ;Save the SP register
                push    cs                      ;Switch to the internal
                pop     ss                      ;  stack
                mov     sp,offset mystack
                sti                             ;Enable interrupts
                cld                             ;Clear direction flag
                push    ax                      ;Save registers
                push    bx
                push    cx
                push    dx
                push    si
                push    bp
                push    ds
                mov     ax,cs                   ;Point DS to the code
                mov     ds,ax                   ;  segment
                assume  ds:code
;
; Save extended error information available from DOS.
;
                cmp     dosversion,030Ah        ;Skip if not 3.10 or later
                jb      save_psp
                push    ds                      ;Save DS
                mov     ah,59h                  ;Get extended error
                sub     bx,bx                   ;  information
                int     21h
                mov     cs:[xerror_ds],ds       ;Save return value of DS
                pop     ds                      ;Set DS to code segment again
                mov     xerror_ax,ax            ;Save return values in
                mov     xerror_bx,bx            ;  XERROR array
                mov     xerror_cx,cx
                mov     xerror_dx,dx
                mov     xerror_si,si
                mov     xerror_di,di
                mov     xerror_es,es
;
; Save the ID for the active process and make this the active one.
;
save_psp:       mov     ah,51h                  ;Get the PSP address of
                int     21h                     ;  the current process
                mov     oldpsp,bx               ;Save it
                mov     ah,50h                  ;Make this the active
                mov     bx,cs                   ;  process by calling
                int     21h                     ;  DOS function 50H
;
; Vector interrupts 1BH, 23H, and 24H to internal routines.
;
                mov     ax,351Bh                ;Hook interrupt 1BH
                int     21h
                mov     word ptr int1Bh,bx
                mov     word ptr int1Bh[2],es
                mov     ax,251Bh
                mov     dx,offset error_exit
                int     21h

                mov     ax,3523h                ;Hook interrupt 23H
                int     21h
                mov     word ptr int23h,bx
                mov     word ptr int23h[2],es
                mov     ax,2523h
                mov     dx,offset error_exit
                int     21h

                mov     ax,3524h                ;Hook interrupt 24H
                int     21h
                mov     word ptr int24h,bx
                mov     word ptr int24h[2],es
                mov     ax,2524h
                mov     dx,offset error_int
                int     21h
;
; Get information about the video environment.
;
                mov     ax,40h                  ;Point ES to the BIOS Data
                mov     es,ax                   ;  Area
                mov     al,es:[49h]             ;Get video mode number in AL
                cmp     al,2                    ;Continue if mode number is
                je      popup1                  ;  2, 3, or 7
                cmp     al,3
                je      popup1
                cmp     al,7
                je      popup1
popup_exit:     jmp     end_popup               ;Exit if it's not

popup1:         mov     ax,es:[4Ah]             ;Get number of columns in AX
                cmp     ax,80                   ;Exit if less than 80 are
                jb      popup_exit              ;  displayed
                shl     ax,1                    ;Compute bytes per video line
                mov     line_length,ax          ;Store it in LINE_LENGTH

                mov     ax,es:[4Eh]             ;Get offset address of the
                mov     video_offset,ax         ;  video buffer
                test    byte ptr es:[63h],40h   ;Branch if this is a color
                jnz     popup2                  ;  video mode
                mov     color_ptr,offset mono_table     ;Prepare for mono-
                mov     video_segment,0B000h            ;  chrome if not
                mov     small_cursor,0B0Ch
                mov     big_cursor,080Ch

popup2:         mov     ax,es:[60h]             ;Get the cursor mode
                mov     cursor_mode,ax          ;Store it
                mov     dx,es:[63h]             ;Place CRTC address in DX
                mov     bl,es:[62h]             ;Get the active page
                sub     bh,bh                   ;  number in BX
                mov     video_page,bl           ;Save it
                shl     bx,1                    ;Multiply it by 2
                mov     ax,es:[bx+50h]          ;Get the cursor address
                mov     cursor_pos,ax           ;Store it
;
; Save the screen and display the popup window.
;               
                mov     ah,01h                  ;Hide the cursor
                mov     cx,2000h
                int     10h
                call    menu
;
; Close the window.
;
                mov     ah,02h                  ;Restore the cursor position
                mov     bh,video_page
                mov     dx,cursor_pos
                int     10h
                mov     ah,01h                  ;Restore the cursor mode
                mov     cx,cursor_mode
                int     10h
;
; Reactivate the old PSP, restore interrupt vectors, and restore extended
; error information before exiting.
;
                mov     ah,50h                  ;Reactivate the PSP that was
                mov     bx,oldpsp               ;  current when this process
                int     21h                     ;  took over

                assume  ds:nothing
                mov     ax,251Bh                ;Restore interrupt 1BH
                lds     dx,cs:[int1Bh]          ;  vector
                int     21h
                mov     ax,2523h                ;Restore interrupt 23H
                lds     dx,cs:[int23h]          ;  vector
                int     21h
                mov     ax,2524h                ;Restore interrupt 24H
                lds     dx,cs:[int24h]          ;  vector
                int     21h

                cmp     cs:[dosversion],030Ah   ;Skip if not 3.10 or later
                jb      end_popup
                mov     ax,cs                   ;Make sure DS points to
                mov     ds,ax                   ;  this segment
                mov     ax,5D0Ah                ;Restore information with
                mov     dx,offset xerror_ax     ;  DOS function 5Dh
                int     21h
;
; Restore register values, restore the stack, and exit.
;
end_popup:      pop     ds
                pop     bp
                pop     si
                pop     dx
                pop     cx
                pop     bx
                pop     ax
                cli
                mov     ss,cs:[ss_register]
                mov     sp,cs:[sp_register]
                sti
                ret
popup           endp

;****************************************************************************
; READKEY reads a keycode from the keyboard buffer.  While it waits for
; a key to be pressed, READKEY generates interrupt 28H over and over to
; give other TSRs a chance to pop up.
;****************************************************************************

readkey         proc    near
                assume  ds:code
                mov     ah,01h                  ;Check keyboard buffer and
                int     16h                     ;  branch if it contains
                jnz     rkey1                   ;  a keycode
                int     28h                     ;Execute an INT 28H and
                jmp     readkey                 ;  loop back
rkey1:          mov     ah,00h                  ;Read the keycode and exit
                int     16h
                ret
readkey         endp

;****************************************************************************
; SAVEREGION saves a region of video memory to a user-specified buffer.
; On entry, CX holds the row and column address of the region's upper left
; corner, DX holds the row and column address of the lower right corner,
; and ES:DI points to the buffer where data will be stored.
;****************************************************************************

saveregion      proc    near
                sub     dh,ch                   ;Compute the number of rows
                inc     dh                      ;  that will be read and
                mov     byte ptr rows,dh        ;  buffered
                sub     dl,cl                   ;Then compute the number of
                inc     dl                      ;  columns
                mov     byte ptr columns,dl

                mov     dx,cx                   ;Place starting address in DX
                call    compute_address         ;Compute the memory address
                mov     si,ax                   ;Transfer result to SI
                push    ds                      ;Save DS
                mov     ds,video_segment        ;Point DS to the video buffer
                assume  ds:nothing

                mov     cx,rows                 ;Load CX with row count
sr_loop:        push    cx                      ;Save count
                push    si                      ;Save starting address
                mov     cx,columns              ;Load CX with column count
                rep     movsw                   ;Buffer one row of text
                pop     si                      ;Retrieve SI
                add     si,line_length          ;Point it to the next row
                pop     cx                      ;Rerieve count
                loop    sr_loop                 ;Loop until all rows are done

                pop     ds                      ;Restore DS and exit
                assume  ds:code
                ret
saveregion      endp

;****************************************************************************
; RESTREGION restores a region of video memory from a user-specified buffer.
; On entry, CX holds the row and column address of the region's upper left
; corner, DX holds the row and column address of the lower right corner,
; and DS:SI points to the buffer the data will be retrieved from.
;****************************************************************************

restregion      proc    near
                sub     dh,ch                   ;Compute the number of rows
                inc     dh                      ;  that will be written
                mov     byte ptr rows,dh        ;
                sub     dl,cl                   ;Then compute the number of
                inc     dl                      ;  columns
                mov     byte ptr columns,dl

                mov     dx,cx                   ;Place starting address in DX
                call    compute_address         ;Compute the memory address
                mov     di,ax                   ;Transfer result to DI
                mov     es,video_segment        ;Point ES to the video buffer

                mov     cx,rows                 ;Load CX with row count
rr_loop:        push    cx                      ;Save count
                push    di                      ;Save starting address
                mov     cx,columns              ;Load CX with column count
                rep     movsw                   ;Buffer one row of text
                pop     di                      ;Retrieve DI
                add     di,line_length          ;Point it to the next row
                pop     cx                      ;Rerieve count
                loop    rr_loop                 ;Loop until all rows are done
                ret
restregion      endp

;****************************************************************************
; DRAWBOX draws a box in text mode using single-line graphics characters.
; On entry, CX holds the row and column address of the upper left corner,
; and DX holds the row and column address of the lower right corner.  AH
; holds the video attribute used to draw the box.
;****************************************************************************

drawbox         proc    near
                sub     dh,ch                   ;Compute the number of rows
                dec     dh                      ;  inside the box
                mov     byte ptr rows,dh
                sub     dl,cl                   ;Then compute the number of
                dec     dl                      ;  columns
                mov     byte ptr columns,dl

                push    ax                      ;Save video attribute
                mov     dx,cx                   ;Place starting address in DX
                call    compute_address         ;Compute the memory address
                mov     di,ax                   ;Transfer result to DI
                mov     es,video_segment        ;Point ES to the video buffer
                pop     ax                      ;Retrieve video attribute
                push    di                      ;Save video buffer address

                mov     al,0DAh                 ;Draw the upper left corner
                stosw                           ;  of the box
                mov     al,0C4h                 ;Draw the upper horizontal
                mov     cx,columns
                rep     stosw
                mov     al,0BFh                 ;Draw the upper right corner
                stosw                           ;  of the box

                sub     di,2                    ;Point DI to the end of the
                add     di,line_length          ;  second row of the box
                mov     cx,rows                 ;Draw right vertical
                call    drawvertical
                mov     al,0D9h                 ;Draw the lower right corner
                stosw                           ;  of the box

                pop     di                      ;Retrieve address
                add     di,line_length          ;Point DI to the second row
                mov     cx,rows                 ;Draw left vertical
                call    drawvertical
                mov     al,0C0h                 ;Draw the lower left corner
                stosw                           ;  of the box

                mov     al,0C4h                 ;Draw the lower horizontal
                mov     cx,columns
                rep     stosw
                ret
drawbox         endp

;****************************************************************************
; DRAWVERTICAL draws a vertical line.  On entry, ES:DI points to the
; location in the video buffer, AH holds the video attribute, and CX
; holds the length of the line in rows.
;****************************************************************************

drawvertical    proc    near
                mov     al,0B3h                 ;Load AL with ASCII code
dv_loop:        stosw                           ;Write one character
                sub     di,2                    ;Point DI to the character
                add     di,line_length          ;  cell on the next row
                loop    dv_loop                 ;Loop until done
                ret
drawvertical    endp

;****************************************************************************
; SHOW_STRING writes an ASCIIZ string at the row and column address
; specified in DH and DL.  On entry, DS:SI points to the string.  Video
; attributes are preserved.
;****************************************************************************

show_string     proc    near
                call    compute_address         ;Compute the memory address
                mov     di,ax                   ;Transfer result to DI
                mov     es,video_segment        ;Point ES to the video buffer
ss_loop:        lodsb                           ;Get a character
                or      al,al                   ;Exit if it's zero
                jz      ss_exit
                stosb                           ;Display it
                inc     di                      ;Skip attribute byte
                jmp     ss_loop                 ;Loop back for more
ss_exit:        ret
show_string     endp

;****************************************************************************
; SHOW_CHAR writes a character only (no attribute) at the row and column
; address specified in DH and DL.  On entry, the character is in AL.
;****************************************************************************

show_char       proc    near
                push    ax                      ;Save the character
                call    compute_address         ;Compute the memory address
                mov     di,ax                   ;Transfer it to DI
                pop     ax                      ;Retrieve the character
                mov     es,video_segment        ;Point ES to the video buffer
                stosb                           ;Display the character
                ret
show_char       endp

;****************************************************************************
; WRITE_ATTR writes the attribute in AL to the row and column address
; passed in DH and DL.  On entry, CX holds the number of consecutive
; attribute bytes to write.
;****************************************************************************

write_attr      proc    near
                push    ax                      ;Save video attribute
                push    cx                      ;Save count
                call    compute_address         ;Compute the memory address
                mov     di,ax                   ;Transfer result to DI
                inc     di                      ;Point DI to attribute bytes
                mov     es,video_segment        ;Point ES to the video buffer
                pop     cx                      ;Retrieve count
                pop     ax                      ;Retrieve video attribute
wa_loop:        stosb                           ;Write one attribute
                inc     di                      ;Advance DI to next one
                loop    wa_loop                 ;Loop until done
wa_exit:        ret
write_attr      endp

;****************************************************************************
; COMPUTE_ADDRESS returns in AX the video buffer address that corresponds
; to the row and column number passed in DH and DL.  Before this procedure
; is called, LINE_LENGTH must contain the number of bytes per video line
; and VIDEO_OFFSET must contain the offset within the video buffer of the
; current video page.
;****************************************************************************

compute_address proc    near
                mov     cl,dl                   ;Save DL in CL
                mov     al,dh                   ;Get starting row in AL
                cbw                             ;Convert byte to word in AX
                mul     line_length             ;Multiply by bytes per row
                mov     dl,cl                   ;Load DL with column number
                shl     dx,1                    ;Multiply starting column by 2
                add     ax,dx                   ;Add it to AX
                add     ax,video_offset         ;Add video buffer offset
                ret
compute_address endp

;****************************************************************************
; STRLEN returns the length of the ASCIIZ string pointed to by DS:SI in CX.
;****************************************************************************

strlen          proc    near
                push    si                      ;Save SI
                sub     cx,cx                   ;Initialize count
strlen1:        lodsb                           ;Get a character
                or      al,al                   ;Exit if it's zero
                jz      strlen_exit
                inc     cx                      ;Otherwise increment the
                jmp     strlen1                 ;  count and loop back
strlen_exit:    pop     si                      ;Restore SI and exit
                ret
strlen          endp

;****************************************************************************
; EDIT_STRING permits a string to be edited.  On entry, DS:SI points to
; the string, DH and DL identify the row and column where the string is
; displayed, and CX holds the maximum number of characters that will be
; accepted.  If AL is 0 when the routine is called, the cursor is
; positioned at the beginning of the string; if it's 1, the cursor is
; positioned at the end.  If BH is 0 when the routine is called, Tab and
; Shift-Tab are ignored; if BH is 1, Tab and Shift-Tab will terminate input
; If BL is 0 when the routine is called, Up- and Down-Arrow are ignored;  if
; BL is 1, Up- and Down-Arrow will terminate input.
;
; On return, CX holds the length of the string, not including the string
; delimiter at the end, and AX holds the key code for the key that was
; pressed to terminate input.
;
; NOTE: The values of DX (the current row and column numbers) and SI (the
; pointer to the current location in the string) are maintained throughout.
;****************************************************************************

chars           dw      ?                       ;Character count
maxchars        dw      ?                       ;Maximum number of characters
stringstart     dw      ?                       ;String starting address
stringend       dw      ?                       ;String ending address
insflag         db      ?                       ;0=Insert off, 1=Insert on
tabflag         db      ?                       ;0=Ignore Tab and Shift-Tab
updnflag        db      ?                       ;0=Ignore Up- and Down-Arrow

edit_string     proc    near
                mov     maxchars,cx             ;Initialize pointers and
                mov     stringstart,si          ;  variables
                mov     tabflag,bh
                mov     updnflag,bl
                mov     insflag,0
                push    ax
                call    strlen
                pop     ax
                mov     chars,cx
                mov     stringend,cx
                add     stringend,si
                or      al,al
                jz      edstr1
                add     dl,cl
                add     si,cx
edstr1:         mov     ah,02h                  ;Position the cursor at
                mov     bh,video_page           ;  the beginning or end
                int     10h                     ;  of the string
                mov     ah,01h                  ;Display the cursor
                mov     cx,small_cursor
                int     10h
;
; Wait for a keypress and process conventional key codes.
;
editkey:        call    readkey                 ;Get a key code
                or      al,al                   ;Branch if it's not an
                jnz     ekey1                   ;  extended code
                jmp     ekey10                  ;Jump if it is

ekey1:          cmp     al,8                    ;Process the Backspace key
                jne     ekey2
                cmp     si,stringstart
                je      editkey
                mov     ah,02h
                mov     bh,video_page
                dec     dl
                int     10h
                dec     si
                jmp     ekey16a

ekey2:          cmp     al,9                    ;Process the Tab key
                jne     ekey3
                cmp     tabflag,0
                je      editkey
ekey2a:         jmp     edstr_exit

ekey3:          cmp     al,13                   ;Process the Enter key
                je      ekey2a

ekey4:          cmp     al,27                   ;Process the Esc key
                je      ekey2a

ekey5:          cmp     insflag,0               ;Process a character key
                jne     ekey6                   ;Branch if the character
                cmp     si,stringend            ;  will be inserted
                je      ekey6
                mov     [si],al
                push    dx
                call    show_char
                pop     dx
ekey5a:         inc     dl
                mov     ah,02h
                mov     bh,video_page
                int     10h
                inc     si
                jmp     editkey

ekey6:          mov     cx,chars                ;Insert a character at the
                cmp     cx,maxchars             ;  current cursor location
                je      editkey
                mov     cx,stringend
                sub     cx,si
                inc     cx
                push    ax
                push    si
                mov     si,stringend
                mov     di,si
                inc     di
                mov     bx,cs
                mov     es,bx
                std
                rep     movsb
                cld
                pop     si
                pop     ax
                mov     [si],al
                inc     stringend
                inc     chars
                push    dx
                push    si
                call    show_string
                pop     si
                pop     dx
                jmp     ekey5a
;
; Process extended key codes.
;
ekey10:         cmp     ah,15                   ;Process Shift-Tab
                jne     ekey11
                cmp     tabflag,0
                je      ekey11a
                jmp     edstr_exit

ekey11:         cmp     ah,71                   ;Process the Home key
                jne     ekey12
                mov     cx,si
                sub     cx,stringstart
                mov     ah,02h
                mov     bh,video_page
                sub     dl,cl
                int     10h
                mov     si,stringstart
ekey11a:        jmp     editkey

ekey12:         cmp     ah,75                   ;Process the Left-Arrow key
                jne     ekey13
                cmp     si,stringstart
                je      ekey12a
                mov     ah,02h
                mov     bh,video_page
                dec     dl
                int     10h
                dec     si
ekey12a:        jmp     editkey

ekey13:         cmp     ah,77                   ;Process the Right-Arrow key
                jne     ekey14
                cmp     si,stringend
                je      ekey13a
                mov     ah,02h
                mov     bh,video_page
                inc     dl
                int     10h
                inc     si
ekey13a:        jmp     editkey

ekey14:         cmp     ah,79                   ;Process the End key
                jne     ekey15
                mov     cx,stringend
                sub     cx,si
                mov     ah,02h
                mov     bh,video_page
                add     dl,cl
                int     10h
                mov     si,stringend
                jmp     editkey

ekey15:         cmp     ah,82                   ;Process the Ins key
                jne     ekey16
                xor     insflag,1
                mov     cx,small_cursor
                cmp     insflag,0
                je      ekey15a
                mov     cx,big_cursor
ekey15a:        mov     ah,01h
                int     10h
                jmp     editkey

ekey16:         cmp     ah,83                   ;Process the Del key
                jne     ekey17
ekey16a:        mov     cx,stringend
                sub     cx,si
                jcxz    ekey16b
                mov     di,si
                mov     bx,cs
                mov     es,bx
                push    si
                inc     si
                rep     movsb
                pop     si
                push    dx
                push    si
                call    show_string
                mov     byte ptr es:[di],32
                pop     si
                pop     dx
                dec     chars
                dec     stringend
ekey16b:        jmp     editkey

ekey17:         cmp     updnflag,0              ;Loop back if Arrow keys
                je      ekey16b                 ;  are ignored
                cmp     ah,72                   ;Terminate input if Up-
                je      edstr_exit              ;  Arrow was pressed
                cmp     ah,80                   ;Terminate input if Down-
                jne     ekey16b                 ;  Arrow was pressed
;
; Hide the cursor, place the string length in CX, and exit.
;
edstr_exit:     push    ax                      ;Save exit character
                mov     ah,01h                  ;Hide the cursor
                mov     ch,20h
                int     10h
                mov     si,stringstart          ;Compute the length of
                call    strlen                  ;  the string
                pop     ax                      ;Retrieve exit character
                ret
edit_string     endp

;****************************************************************************
; COPYZ copies an ASCIIZ string from one location to another.  On entry,
; DS:SI points to the string and ES:DI points to its new location.
;****************************************************************************

copyz           proc    near
                lodsb                           ;Get a character
                stosb                           ;Copy it to the destination
                or      al,al                   ;Loop back for more if it
                jnz     copyz                   ;  wasn't a zero
                ret
copyz           endp

;****************************************************************************
; Menu window of choices
;****************************************************************************

exitcode        dw      ?
spaces          db      10 dup (32),0

menu            proc    near
;
; Turn on display
;
                mov     ah,28                   ;load value to turn display on
                int     64h                     ;interrupt RadioDRV
                mov     bx,color_ptr            ;Place attributes in AH,
                mov     ah,border_color         ;  AL, and BH
                mov     al,window_color_2
                mov     bh,shadow_color
                mov     cx,window_start         ;Load CX with the window's
                                                ;  starting coordinates
                mov     dx,cx                   ;Load DX with the window's
                add     dh,20                   ;  ending coordinates
                add     dl,33
                push    cx                      ;Save window coordinates
                push    dx
                push    cs                      ;Point ES:DI to the buffer
                pop     es                      ;  for screen data
                mov     di,offset screenbuffer1
                call    openwindow              ;Display the window

                mov     dx,window_start         ;Compute row and column
                add     dh,1                    ;  address to begin
                add     dl,2                    ;  writes

                push    dx                      ;Save row and column
                mov     si,offset title1        ;Point DS:SI to the text
                call    show_string             ;Display title string
                pop     dx                      ;Retrieve row and column

                inc     dh                      ;Display line1
                push    dx
                mov     si,offset line
                call    show_string
                pop     dx

                inc     dh                      ;Display "Offset    :"
                push    dx
                mov     si,offset head1
                call    show_string
                pop     dx

                inc     dh                      ;Display "Base Freq :"
                push    dx
                mov     si,offset head2
                call    show_string
                pop     dx

                inc     dh                      ;Display "Doppler   :"
                push    dx
                mov     si,offset head3
                call    show_string
                pop     dx

                inc     dh                      ;Display "Cur Freq  :"
                push    dx
                mov     si,offset head4
                call    show_string
                pop     dx

                inc     dh                      ;Display line2
                push    dx
                mov     si,offset line
                call    show_string
                pop     dx

                inc     dh                      ;Display "-10 Hz"
                push    dx
                mov     si,offset funkey1
                call    show_string
                pop     dx

                inc     dh                      ;Display "+10 Hz"
                push    dx
                mov     si,offset funkey2
                call    show_string
                pop     dx

                inc     dh                      ;Display "-100 Hz"
                push    dx
                mov     si,offset funkey3
                call    show_string
                pop     dx

                inc     dh                      ;Display "+100 Hz"
                push    dx
                mov     si,offset funkey4
                call    show_string
                pop     dx

                inc     dh                      ;Display "-1000 Hz"
                push    dx
                mov     si,offset funkey5
                call    show_string
                pop     dx

                inc     dh                      ;Display "+1000 Hz"
                push    dx
                mov     si,offset funkey6
                call    show_string
                pop     dx

                inc     dh                      ;Display "Zero Offset"
                push    dx
                mov     si,offset funkey7
                call    show_string
                pop     dx

                inc     dh                      ;Display "Toggle Freq Updating"
                push    dx
                mov     si,offset funkey8
                call    show_string
                pop     dx

                inc     dh                      ;Display "Toggle Band Display
                push    dx
                mov     si,offset funkey9
                call    show_string
                pop     dx

                inc     dh                      ;Display "Set Base Frequency"
                push    dx
                mov     si,offset funkey10
                call    show_string
                pop     dx

                inc     dh                      ;Display line3
                push    dx
                mov     si,offset line
                call    show_string
                pop     dx

                inc     dh                      ;Display inputline
                push    dx
                mov     si,offset inputline
                call    show_string
                pop     dx

                add     dl,20                   ;Set input line attributes
                mov     bx,color_ptr            ;Place attributes in AL
                mov     al,input_color
                mov     cx,10                   ;Width of input area
                call    write_attr
;
; Process keystrokes with the window displayed.
;
getkey:         mov     bx,color_ptr            ;Draw the selection bar at
                mov     al,hilite_color_2       ;  the location addressed
                mov     cx,32                   ;  by INDEX
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr

getkey1:        call    readkey                 ;Get a keystroke
                or      al,al                   ;Branch if it's an extended
                jnz     enterkey                ;  key code
                jmp     extended
;
; Output control codes if ENTER was pressed.
;
enterkey:       cmp     al,13                   ;Check for ENTER keycode
                jne     escape
                cmp     index,9                 ;Is index at F10?
                jne     enterkey1               ;Branch if it isn't
                jmp     slash1                  ;Else treat it like a slash
enterkey1:      mov     ah,index                ;Set command code'
                add     ah,16                   ;Set command code (index + 16)
                jmp     output_updcmd           ;Output indexed update command
;
; Close the window and exit if ESC was pressed.
;
escape:         cmp     al,27                   ;Check for ESC keycode
                jne     equal                   ;
                jmp     close                   ;Branch to exit routines
;
; Zero frequency offset if '=' key was pressed
;
equal:          cmp     al,"="                  ;Check for '='
                jne     letter_u                ;Branch if not an equal sign
                mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                mov     index,6                 ;Set INDEX
                mov     ah,22                   ;Set command code (index + 16)
                jmp     output_updcmd
;
; Toggle frequency update if 'U' key was pressed
;
letter_u:       cmp     al,"u"                  ;Check for lower case 'u'
                jne     letter_u1               ;Branch if not a 'u'
                jmp     letter_u2
letter_u1:      cmp     al,'U'                  ;Check for upper case 'U'
                jne     letter_d                ;Branch if not a 'U'
letter_u2:      mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                mov     index,7                 ;Set INDEX
                mov     ah,23                   ;Set command code (index + 16)
                jmp     output_updcmd
;
; Toggle band displayed if 'D' key was pressed
;
letter_d:       cmp     al,"d"                  ;Check for lower case 'd'
                jne     letter_d1               ;Branch if not a 'd'
                jmp     letter_d2
letter_d1:      cmp     al,'D'                  ;Check for upper case 'D'
                jne     letter_o                ;Branch if not a 'D'
letter_d2:      mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                mov     index,8                 ;Set INDEX
                mov     ah,24                   ;Set command code (index + 16)
                jmp     output_updcmd
;
; Toggle one-shot frequency update if 'O' key was pressed
;
letter_o:       cmp     al,"o"                  ;Check for lower case 'o'
                jne     letter_o1               ;Branch if not a 'o'
                jmp     letter_o2
letter_o1:      cmp     al,'O'                  ;Check for upper case 'O'
                jne     letter_b                ;Branch if not a 'O'
letter_o2:      mov     ah,30                   ;Set command code
                jmp     output_updcmd
;
; Toggle through band combinations if 'B' key was pressed
;
letter_b:       cmp     al,"b"                  ;Check for lower case 'b'
                jne     letter_b1               ;Branch if not a 'b'
                jmp     letter_b2
letter_b1:      cmp     al,'B'                  ;Check for upper case 'B'
                jne     letter_i                ;Branch if not a 'B'
letter_b2:      mov     ah,31                   ;Set command code
                jmp     output_updcmd
;
; Toggle offset inverting if 'I' key was pressed
;
letter_i:       cmp     al,"i"                  ;Check for lower case 'i'
                jne     letter_i1               ;Branch if not a 'i'
                jmp     letter_i2
letter_i1:      cmp     al,'I'                  ;Check for upper case 'I'
                jne     slash                   ;Branch if not a 'I'
letter_i2:      mov     ah,32                   ;Set command code
                jmp     output_updcmd
;
; Enter new base frequency if the slash key was pressed.
;
slash:          cmp     al,"/"                  ;Check for slash character
                jne     otherwise               ;Branch if not slash
slash1:         jmp     get_string
;
; One-shot frequency update if any other key was pressed
;
otherwise:      mov     ah,40                   ;Load comand value for one-shot
                jmp     output_updcmd
;
; Input a new base frequency string.
;
get_string:     mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                mov     index,9                 ;Set INDEX

                mov     al,input_color          ;Set attributes of input area
                mov     cx,10                   ;Length of input area
                mov     dx,window_start         ;Upper-left corner
                add     dh,19                   ;Add rows
                add     dl,22                   ;Add columns
                push    dx                      ;Save it
                call    write_attr
                pop     dx
                push    dx                      ;Display current string in area
                mov     si,offset freqstring
                call    show_string
                pop     dx
                mov     si,[offset freqstring]  ;Get offset address of
                                                ;  the string to be edited
                push    si                      ;Save it
                mov     di,offset inbuffer      ;Point DI to input buffer
                mov     bx,cs                   ;Point ES to this segment
                mov     es,bx
                call    copyz                   ;Copy string to input buffer

                mov     al,01h                  ;Prepare for call to
                mov     bx,01h                  ;  EDIT_STRING
                mov     cx,10
                mov     dx,window_start
                add     dh,19
                add     dl,22
                mov     si,offset inbuffer
                call    edit_string             ;Let the string be edited

                pop     di                      ;Retrieve its address in DI
                cmp     al,27                   ;Branch if the Esc key was
                jne     getstr1                 ;  not pressed
                mov     dx,window_start         ;Otherwise redisplay the
                add     dh,19                   ;  original string and return
                add     dl,22                   ;  to the menu
                push    dx
                push    di
                mov     si,offset spaces        ;Blank the input area first
                call    show_string
                pop     si
                pop     dx
                call    show_string             ;Then rewrite the string
                jmp     getkey

getstr1:        mov     bx,ax                   ;Save exit code in BX
                mov     si,offset inbuffer      ;Copy the new string to the
                mov     ax,cs                   ;  original string address
                mov     es,ax
                call    copyz
;
; Move the new base_freq string to RadioDRV.  Then do the interrupt.
;
                push    ds                      ;save DS
                push    cs                      ;set DS to CS
                pop     ds
                mov     si,offset inbuffer      ;point SI to text
                mov     al,10                   ;pass the character count
                mov     index,9                 ;Set INDEX
                mov     ah,25                   ;Set command code (index + 16)
                int     64h                     ;interrupt RadioDRV
                pop     ds                      ;restore DS
                jmp     getkey                  ;return to input loop
;
; Process extended keycodes.
;
extended:       cmp     ah,59                   ;Process F1
                jb      functionkeys2
                cmp     ah,68
                ja      up
                push    ax                      ;Save extended keycode
                mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                pop     ax                      ;Recover keycode
                sub     ah,59                   ;Calculate new INDEX
                mov     index,ah
                cmp     ah,9                    ;Is it F10?
                jne     functionkeys1           ;Branch if it isn't
                jmp     slash1                  ;Else treat it like a slash
functionkeys1:  add     ah,16                   ;Set command code (index + 16)
                jmp     output_updcmd           ;Output update command
functionkeys2:  jmp     getkey
;
; Move the menu bar up one line if up-arrow was pressed.
;
up:             cmp     ah,72                   ;Process the Up-Arrow key
                jne     down
up1:            mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                dec     index                   ;Decrement index
                cmp     index,0FFh
                jne     up2
                mov     index,9
up2:            jmp     getkey                  ;Return to the input loop
;
; Move the menu bar down one line if down-arrow was pressed.
;
down:           cmp     ah,80                   ;Process the Down-Arrow key
                je      down1
                jmp     getkey1
down1:          mov     bx,color_ptr            ;Erase the selection bar
                mov     al,window_color_2
                mov     cx,32
                mov     dx,window_start
                add     dh,8
                add     dh,index
                add     dl,1
                call    write_attr
                inc     index                   ;Increment index
                cmp     index,10
                jne     down2
                mov     index,0
down2:          jmp     getkey                  ;Return to the input loop
;
; Output the update command; command code is in AH
;
output_updcmd:  int     64h                     ;interrupt RadioDRV
                jmp     getkey                  ;return to input loop
;
; Close the menu window.  Turn off frequency displays
;
close:
                mov     ah,29                   ;load value to turn display off
                int     64h                     ;interrupt RadioDRV
                pop     dx                      ;Close the window by
                pop     cx                      ;  restoring what was
                add     dx,0101h                ;  under it
                mov     si,offset screenbuffer1
                call    restregion
menu_end:       ret
menu            endp

;****************************************************************************
; OPENWINDOW opens a window for input.  On entry CH and CL hold the
; coordinates of the window's upper left corner; DH and DL hold the
; coordinates of the lower right corner; AH holds the window's border
; color; AL holds the color of its interior; BH holds the drop shadow
; color; and ES:DI points to the buffer where video data will be stored.
;****************************************************************************

ul_col          db      ?                       ;Upper left column
ul_row          db      ?                       ;Upper left row
lr_col          db      ?                       ;Lower right column
lr_row          db      ?                       ;Lower right row
border_attr     db      ?                       ;Border attribute
window_attr     db      ?                       ;Window attribute
shadow_attr     db      ?                       ;Shadow attribute

openwindow      proc    near
                mov     word ptr ul_col,cx      ;Save window coordinates
                mov     word ptr lr_col,dx
                mov     border_attr,ah          ;Store video attributes
                mov     window_attr,al
                mov     shadow_attr,bh

                add     dx,0101h                ;Include drop shadow
                call    saveregion              ;Save the screen

                mov     ah,border_attr          ;Retrieve attribute
                mov     cx,word ptr ul_col      ;Retrieve window coordinates
                mov     dx,word ptr lr_col
                call    drawbox                 ;Draw the window border

                mov     bh,window_attr          ;Retrieve attribute
                mov     cx,word ptr ul_col      ;Retrieve window coordinates
                add     cx,0101h
                mov     dx,word ptr lr_col
                sub     dx,0101h
                mov     ax,0600h                ;Blank the interior of the
                int     10h                     ;  window

                mov     al,shadow_attr          ;Retrieve attribute
                mov     cl,lr_col               ;Compute length of the
                sub     cl,ul_col               ;  horizontal portion
                inc     cl                      ;  of the drop shadow
                sub     ch,ch                   ;Byte to word in CX
                mov     dh,lr_row               ;Load DX with the starting
                mov     dl,ul_col               ;  row and column address
                add     dx,0101h
                call    write_attr              ;Write the attribute

                mov     cl,lr_row               ;Compute length of the
                sub     cl,ul_row               ;  vertical portion of
                inc     cl                      ;  the drop shadow
                sub     ch,ch                   ;Byte to word in CX
                push    cx                      ;Save it
                mov     dh,ul_row               ;Load DX with the starting
                mov     dl,lr_col               ;  row and column address
                add     dx,0101h
                call    compute_address         ;Compute the address
                mov     di,ax                   ;Transfer offset to DI
                inc     di                      ;Point DI to attribute
                mov     es,video_segment        ;Point ES to video segment
                pop     cx                      ;Retrieve length
                mov     al,shadow_attr          ;Retrieve attribute
open_loop:      stosb                           ;Write one attribute
                add     di,line_length          ;Point DI to next attribute
                dec     di                      ;  byte
                loop    open_loop               ;Loop until done
                ret
openwindow      endp

;****************************************************************************
; Buffer areas to be used after installation.
;****************************************************************************

screenbuffer1   =       $                       ;Screen buffer (1920 bytes)
mystack         =       $ + 1920                ;Internal stack (512 bytes)
databuffer      =       $ + 2432                ;Data buffer

;****************************************************************************
; Data that will be discarded when the program becomes memory-resident.
;****************************************************************************

helpmsg         db      "Installs a popup control program for RadioDRV."
                db      13,10,13,10
                db      "RadioPOP [/M]"
                db      13,10,13,10
                db      "  /M        Use monochrome video attributes."
                db      13,10,13,10
                db      "Once RadioPOP is installed, pressing Alt and the "
                db      "right Shift key pops ",13,10
                db      "up the RadioPOP control window.",13,10,"$"

errmsg1         db      "Syntax: RadioPOP  [/M]",13,10,"$"
errmsg2         db      "Requires DOS 3.0 or higher",13,10,"$"
errmsg3         db      "Program could not be installed",13,10,"$"
errmsg4         db      "Program is already installed",13,10,"$"
errmsg5         db      "RadioDRV not installed.  RadioPOP installation halted."
                db      13,10,"$"

msg1            db      "RadioPOP 1.1 installed",13,10
                db      "Hotkey is Alt-Right Shift",13,10,"$"

eol_flag        db      0                       ;1=End of line reached

;****************************************************************************
; INIT makes the program resident in memory.
;****************************************************************************

                assume  cs:code,ds:code

init            proc    near
                cld                             ;Clear direction flag
                mov     si,81h                  ;Point SI to command line
                call    scanhelp                ;Scan for "/?" switch
                jnc     checkver                ;Branch if not found
                mov     ah,09h                  ;Display help text and exit
                mov     dx,offset helpmsg       ;  with ERRORLEVEL=0
                int     21h
                mov     ax,4C00h
                int     21h
;
; Check the DOS version and terminate if the program is already installed.
;
checkver:       mov     dx,offset errmsg2       ;Get the DOS version number
                mov     ah,30h
                int     21h
                mov     byte ptr dosversion,ah  ;Save it
                mov     byte ptr dosversion[1],al
                cmp     al,3                    ;Exit if version number is
                jae     checkins                ;  less than 3.0

error:          mov     ah,09h                  ;Display error message and
                int     21h                     ;  exit with ERRORLEVEL=1
                mov     ax,4C01h
                int     21h

checkins:       push    es                      ;Save ES
                call    check_install           ;Get installed status
                pop     es                      ;Restore ES
                mov     dx,offset errmsg4       ;Error if the program is
                jc      error                   ;  already installed
;
; Verify that RadioDRV is already installed
;
                push    es                      ;Save ES
                mov     dx,offset errmsg5       ;Initialize error pointer
                mov     ax,3564h                ;Get interrupt 64h vector
                int     21h
                mov     di,bx                   ;Vector returned in ES:BX
                add     di,0Ah                  ;Add signature area offset
                mov     si,offset signature1    ;Put sig constant addr in DS:SI
                mov     cx,8                    ;Signature length
                cld
                repe    cmpsb                   ;Compare them
                pop     es                      ;Restore ES
                jne     error
                mov     si,81h                  ;Reset SI
;
; Parse the command line.
;
        
parse:          call    findchar                ;Find first parameter
                jc      endparse                ;Branch if none found
                mov     dx,offset errmsg1       ;Initialize error pointer
                cmp     byte ptr [si],"/"       ;Branch if it's a "/"
                je      slashm                  ;  character
                jmp     error1
;
; Process a /M switch.
;
slashm:         mov     dx,offset errmsg1       ;Initialize error pointer
                inc     si                      ;Skip the "/" character
                lodsb                           ;Get a character
                and     al,0DFh                 ;Capitalize it
                cmp     al,"M"                  ;Error if the character is
                jne     error1                  ;  not an "M"
                push    si                      ;Save SI
                mov     si,offset mono_table    ;Point SI to mono attributes
                mov     di,offset color_table   ;Point DI to color attributes
                mov     cx,11                   ;Initalize counter
                rep     movsb                   ;Copy mono to color
                pop     si                      ;Restore SI
                jmp     parse                   ;Reenter the parsing loop

endparse:       jmp     install
error1:         jmp     error

;
; Install the program.
;
install:        call    mplex_id                ;Find a multiplex ID number
                mov     dx,offset errmsg3       ;Error if none available
                jc      error1
                mov     prog_id,ah              ;Save the ID number

                mov     ah,34h                  ;Get and save the address of
                int     21h                     ;  the InDOS flag
                mov     word ptr indos,bx
                mov     word ptr indos[2],es

                push    ds                      ;Save DS
                mov     ax,5D06h                ;Get and save the address of
                int     21h                     ;  the critical error flag
                mov     word ptr cs:[errorflag],si
                mov     word ptr cs:[errorflag+2],ds
                pop     ds                      ;Restore DS
                mov     dx,offset errmsg3       ;Error if function returned
                jc      error1                  ;  with carry set

                mov     si,offset databuffer-1  ;Zero the byte just before
                mov     byte ptr [si],0         ;  the data buffer

                mov     ah,12h                  ;Test for EGA/VGA
                mov     bl,10h
                int     10h
                cmp     bl,10h                  ;EGA/VGA installed if BL
                jne     set_vectors             ;  returned changed
                mov     egaflag,0               ;Zero video adapter flag

set_vectors:
                mov     ax,3509h                ;Hook interrupt 09H
                int     21h
                mov     word ptr int09h,bx
                mov     word ptr int09h[2],es
                mov     ax,2509h
                mov     dx,offset kb_int
                int     21h

                mov     ax,3510h                ;Hook interrupt 10H
                int     21h
                mov     word ptr int10h,bx
                mov     word ptr int10h[2],es
                mov     ax,2510h
                mov     dx,offset video_int
                int     21h

                mov     ax,3528h                ;Hook interrupt 28H
                int     21h
                mov     word ptr int28h,bx
                mov     word ptr int28h[2],es
                mov     ax,2528h
                mov     dx,offset dosidle_int
                int     21h

                mov     ax,352Fh                ;Hook interrupt 2FH
                int     21h
                mov     word ptr int2Fh,bx
                mov     word ptr int2Fh[2],es
                mov     ax,252Fh
                mov     dx,offset mplex_int
                int     21h

                mov     ah,49h                  ;Get the segment address of
                mov     es,ds:[2Ch]             ;  the environment block
                int     21h                     ;  and free the segment

                mov     ah,09h                  ;Display message verifying
                mov     dx,offset msg1          ;  the installation
                int     21h

                mov     ax,3100h                ;Terminate with function 31H
                mov     dx,offset databuffer+15 ;Compute amount of memory to
                mov     cl,4
                shr     dx,cl
                int     21h
init            endp

;****************************************************************************
; FINDCHAR advances SI to the next non-space or non-comma character.
; On return, carry set indicates EOL was encountered.
;****************************************************************************

findchar        proc    near
                lodsb                           ;Get the next character
                cmp     al,20h                  ;Loop if space
                je      findchar
                cmp     al,2Ch                  ;Loop if comma
                je      findchar
                dec     si                      ;Point SI to the character
                cmp     al,0Dh                  ;Exit with carry set if end
                je      eol                     ;  of line is reached

                clc                             ;Clear carry and exit
                ret

eol:            stc                             ;Set carry and exit
                ret
findchar        endp

;****************************************************************************
; SCANHELP scans the command line for a /? switch.  If found, carry returns
; set and SI contains its offset.  If not found, carry returns clear.
;****************************************************************************

scanhelp        proc    near
                push    si                      ;Save SI
scanloop:       lodsb                           ;Get a character
                cmp     al,0Dh                  ;Exit if end of line
                je      scan_exit
                cmp     al,"?"                  ;Loop if not "?"
                jne     scanloop
                cmp     byte ptr [si-2],"/"     ;Loop if not "/"
                jne     scanloop

                add     sp,2                    ;Clear the stack
                sub     si,2                    ;Adjust SI
                stc                             ;Set carry and exit
                ret

scan_exit:      pop     si                      ;Restore SI
                clc                             ;Clear carry and exit
                ret
scanhelp        endp

;****************************************************************************
; CHECK_INSTALL returns carry set if the program is already installed,
; carry clear if it's not.  If carry returns set, AH holds the program's
; multiplex ID number.
;****************************************************************************

check_install   proc    near
                mov     ax,8000h                ;Initialize AH and AL
                mov     cx,80h                  ;Initialize count

chinst1:        push    ax                      ;Save AX and CX
                push    cx
                sub     di,di                   ;Set ES and DI to 0
                mov     es,di
                int     2Fh                     ;Interrupt 2Fh
                cmp     al,0FFh                 ;Nothing here if AL isn't
                jne     chinst2                 ;  equal to FFH

                mov     si,offset signature     ;See if program signature
                mov     cx,13                   ;  appears at the address
                repe    cmpsb                   ;  returned in ES:DI
                jne     chinst2                 ;Branch if it does not

                pop     cx                      ;Clear the stack and exit
                pop     ax                      ;  with carry set
                stc
                ret

chinst2:        pop     cx                      ;Retrieve AX and CX
                pop     ax
                inc     ah                      ;Next multiplex ID
                loop    chinst1                 ;Loop until done

                clc                             ;Exit with carry clear
                ret
check_install   endp

;****************************************************************************
; MPLEX_ID searches for an unused multiplex ID number.  If one is found,
; it is returned in AH with carry clear.  Carry set means no multiplex
; ID numbers are currently available.
;****************************************************************************

mplex_id        proc    near
                mov     ax,8000h                ;Initialize AH and AL
                mov     cx,80h                  ;Initialize count

mxid1:          push    ax                      ;Save AX and CX
                push    cx
                int     2Fh                     ;Interrupt 2Fh
                or      al,al                   ;Branch if AL=0
                jz      mxid2
                pop     cx                      ;Retrieve AX and CX
                pop     ax
                inc     ah                      ;Increment ID number
                loop    mxid1                   ;Loop until done

                stc                             ;Exit with carry set
                ret

mxid2:          pop     cx                      ;Clear the stack and exit
                pop     ax                      ;  with carry clear
                clc
                ret
mplex_id        endp

code            ends
                end     begin
